<?php

use App\Entity\DrinkType;
use App\Enum\SystemSettingKey;
use App\Repository\DrinkTypeRepository;
use App\Service\ConfigurationService;
use App\Service\DrinkTypeService;
use Doctrine\ORM\EntityManagerInterface;

test('getAllDrinkTypes returns all drink types', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);

    // Act
    $drinkTypes = $drinkTypeService->getAllDrinkTypes();

    // Assert
    expect($drinkTypes)->toBeArray();
});

test('getDrinkTypeById returns correct drink type', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Test Drink Type');
    $drinkType->setDescription('Test Description');
    $drinkType->setDesiredStock(5);
    $em->persist($drinkType);
    $em->flush();

    $id = $drinkType->getId();

    // Act
    $retrievedDrinkType = $drinkTypeService->getDrinkTypeById($id);

    // Assert
    expect($retrievedDrinkType)->toBeInstanceOf(DrinkType::class);
    expect($retrievedDrinkType->getId())->toBe($id);
    expect($retrievedDrinkType->getName())->toBe('Test Drink Type');
});

test('getDrinkTypeById returns null for non-existent id', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $nonExistentId = 9999;

    // Act
    $drinkType = $drinkTypeService->getDrinkTypeById($nonExistentId);

    // Assert
    expect($drinkType)->toBeNull();
});

test('getDrinkTypeByName returns correct drink type', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Test Drink Type By Name');
    $drinkType->setDescription('Test Description');
    $drinkType->setDesiredStock(5);
    $em->persist($drinkType);
    $em->flush();

    // Act
    $retrievedDrinkType = $drinkTypeService->getDrinkTypeByName('Test Drink Type By Name');

    // Assert
    expect($retrievedDrinkType)->toBeInstanceOf(DrinkType::class);
    expect($retrievedDrinkType->getName())->toBe('Test Drink Type By Name');
});

test('getDrinkTypeByName returns null for non-existent name', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $nonExistentName = 'Non-Existent Drink Type';

    // Act
    $drinkType = $drinkTypeService->getDrinkTypeByName($nonExistentName);

    // Assert
    expect($drinkType)->toBeNull();
});

test('createDrinkType creates new drink type with provided values', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $name = 'New Drink Type';
    $description = 'New Description';
    $desiredStock = 10;

    // Act
    $drinkType = $drinkTypeService->createDrinkType($name, $description, $desiredStock);

    // Assert
    expect($drinkType)->toBeInstanceOf(DrinkType::class);
    expect($drinkType->getName())->toBe($name);
    expect($drinkType->getDescription())->toBe($description);
    expect($drinkType->getDesiredStock())->toBe($desiredStock);
});

test('createDrinkType creates new drink type with default desired stock', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $configService = $this->getContainer()->get(ConfigurationService::class);
    $name = 'New Drink Type Default Stock';
    $description = 'New Description';

    // Set default desired stock in configuration
    $defaultDesiredStock = '15';
    $configService->setConfigValue(SystemSettingKey::DEFAULT_DESIRED_STOCK, $defaultDesiredStock);

    // Act
    $drinkType = $drinkTypeService->createDrinkType($name, $description);

    // Assert
    expect($drinkType)->toBeInstanceOf(DrinkType::class);
    expect($drinkType->getName())->toBe($name);
    expect($drinkType->getDescription())->toBe($description);
    expect($drinkType->getDesiredStock())->toBe((int)$defaultDesiredStock);
});

test('createDrinkType throws exception when drink type with same name exists', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $name = 'Duplicate Drink Type';

    // Create a drink type with the same name
    $drinkTypeService->createDrinkType($name);

    // Act & Assert
    expect(fn() => $drinkTypeService->createDrinkType($name))
        ->toThrow(InvalidArgumentException::class);
});

test('updateDrinkType updates drink type properties', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Original Drink Type');
    $drinkType->setDescription('Original Description');
    $drinkType->setDesiredStock(5);
    $em->persist($drinkType);
    $em->flush();

    $newName = 'Updated Drink Type';
    $newDescription = 'Updated Description';
    $newDesiredStock = 15;

    // Act
    $updatedDrinkType = $drinkTypeService->updateDrinkType(
        $drinkType,
        $newName,
        $newDescription,
        $newDesiredStock
    );

    // Assert
    expect($updatedDrinkType)->toBeInstanceOf(DrinkType::class);
    expect($updatedDrinkType->getName())->toBe($newName);
    expect($updatedDrinkType->getDescription())->toBe($newDescription);
    expect($updatedDrinkType->getDesiredStock())->toBe($newDesiredStock);
});

test('updateDrinkType throws exception when updating to existing name', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create two drink types
    $drinkType1 = new DrinkType();
    $drinkType1->setName('First Drink Type');
    $em->persist($drinkType1);

    $drinkType2 = new DrinkType();
    $drinkType2->setName('Second Drink Type');
    $em->persist($drinkType2);
    $em->flush();

    // Act & Assert
    expect(fn() => $drinkTypeService->updateDrinkType($drinkType2, 'First Drink Type'))
        ->toThrow(InvalidArgumentException::class);
});

test('updateDrinkType only updates provided properties', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Partial Update Drink Type');
    $drinkType->setDescription('Original Description');
    $drinkType->setDesiredStock(5);
    $em->persist($drinkType);
    $em->flush();

    $newDescription = 'Updated Description';

    // Act - only update description
    $updatedDrinkType = $drinkTypeService->updateDrinkType(
        $drinkType,
        null,
        $newDescription,
        null
    );

    // Assert
    expect($updatedDrinkType)->toBeInstanceOf(DrinkType::class);
    expect($updatedDrinkType->getName())->toBe('Partial Update Drink Type');
    expect($updatedDrinkType->getDescription())->toBe($newDescription);
    expect($updatedDrinkType->getDesiredStock())->toBe(5);
});

test('deleteDrinkType removes drink type', function () {
    // Arrange
    $drinkTypeService = $this->getContainer()->get(DrinkTypeService::class);
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Drink Type To Delete');
    $em->persist($drinkType);
    $em->flush();

    $id = $drinkType->getId();

    // Act
    $drinkTypeService->deleteDrinkType($drinkType);

    // Assert
    $deletedDrinkType = $drinkTypeService->getDrinkTypeById($id);
    expect($deletedDrinkType)->toBeNull();
});
